//
//  DeviceView.swift
//  WELDeviceExample
//

import SwiftUI
import WELDeviceLibrary

struct DeviceDetails: View {
    @EnvironmentObject var managerWrapper: ManagerWrapper
    @EnvironmentObject var deviceWrapper: DeviceWrapper
    
    var device: SFTDevice {
        deviceWrapper.device
    }
    
    var body: some View {
        VStack(alignment: .leading) {
            HStack{
                Text("SFT Device")
                    .bold()
                Spacer()
                Text(deviceWrapper.device.isConnected ? "Connected" : "Disconnected")
                    .bold()
            }
            
            Text(deviceWrapper.id)
                .font(.subheadline)
                .padding(.bottom)
            
            // Connected
            if deviceWrapper.device.isConnected {
                Group {
                    Text("Firmware:")
                        .bold()
                    Text(deviceWrapper.firmwareDetails)
                        .padding(.bottom)
                    
                    Text("Serial No:")
                        .bold()
                    Text(deviceWrapper.serialNoDetails)
                        .padding(.bottom)
                    
                    Text("Battery:")
                        .bold()
                    Text(deviceWrapper.batteryDetails)
                        .padding(.bottom)
                    
                    Text("Stats:")
                        .bold()
                    Text(deviceWrapper.statsDetails)
                        .padding(.bottom)
                    
                    Text("Last Measurement:")
                        .bold()
                    Text(deviceWrapper.measurementDetails)
                        .padding(.bottom)
                }
                
                Group {
                    Button(action: getFirmware) {
                        Text("Get Firmware")
                    }
                    Button(action: getSerialNumber) {
                        Text("Get Serial Number")
                    }
                    Button(action: getBatteryDetails) {
                        Text("Get Battery Details")
                    }
                    Button(action: getStats) {
                        Text("Get Stats")
                    }
                    Button(action: getLastMeasurement) {
                        Text("Get Last Measurement")
                    }
                    Button(action: takeMeasurement) {
                        Text("Take Measurement")
                    }
                    Button(action: disconnect) {
                        Text("Disconnect")
                    }
                }
                
            // Disconnected
            } else {
                Button(action: connect) {
                    Text("Connect")
                }
            }
        }
        .padding(.all)
    }
    
    func getFirmware() {
        device.readFirmwareVersion()
    }
    
    func getSerialNumber() {
        device.readSerialNumber()
    }
    
    func getBatteryDetails() {
        device.readBatteryPercentage()
    }
    
    func getStats() {
        device.readStats()
    }
    
    func getLastMeasurement() {
        device.readLastMeasurement()
    }
    
    func takeMeasurement() {
        device.takeMeasurement()
    }
    
    func connect() {
        managerWrapper.manager?.connect(device)
    }
    
    func disconnect() {
        managerWrapper.manager?.disconnect(device)
    }
}

struct DeviceDetails_Previews: PreviewProvider {
    static var previews: some View {
        DeviceDetails()
            .environmentObject(DeviceWrapper(MockDevice()))
    }
}
