//
//  DeviceWrapper.swift
//  WELDeviceExample
//

import Foundation
import WELDeviceLibrary

class DeviceWrapper : SFTDeviceDelegate, ObservableObject, Identifiable {
    var device: SFTDevice
    
    @Published var isReady: Bool = false
    @Published var firmwareDetails: String = ""
    @Published var batteryDetails: String = ""
    @Published var statsDetails: String = ""
    @Published var measurementDetails: String = ""
    @Published var serialNoDetails: String = ""
    
    public var id: String {
        device.id
    }
    
    public init(_ device: SFTDevice) {
        self.device = device
        device.setDelegate(self)
    }
    
    func onBatteryResponse(_ response: WELDeviceLibrary.BatteryResponse) {
        var details = ""
        
        switch (response.getBatteryState()) {
            case .unknown:
                details = "UNKNOWN"
            case .discharging:
                details = "DISCHARGING"
            case .charging:
                details = "CHARGING"
            case .charged:
                details = "CHARGED"
        }
        
        if let percentage = response.getBatteryPercentage() {
            details = details.isEmpty ? String(percentage) : "\(details), \(percentage)"
        }
        
        batteryDetails = details
    }
    
    func onFirmwareResponse(_ response: WELDeviceLibrary.FirmwareResponse) {
        firmwareDetails = response.getFirmware() ?? ""
    }
    
    func onStatsResponse(_ response: WELDeviceLibrary.DeviceStatsResponse) {
        statsDetails = [
            response.getTotalMeasurements(),
            response.getUnused(),
            response.getPowerCycles(),
            response.getUSBPlugCycles()
        ].map({ $0.map(String.init) ?? "-" }).joined(separator: ", ")
    }
    
    func onMeasurementResponse(_ optionalResponse: WELDeviceLibrary.MeasurementResponse?) {
        guard let response = optionalResponse else {
            measurementDetails = "-"
            return
        }
        if let errorMsg = response.getErrorMessage() {
            measurementDetails = "\(response.getIsValidMeasurement()), \(errorMsg)"
            return
        }
        measurementDetails = "\(response.getIsValidMeasurement()), " + [
            response.getKGForce(),
            response.getGForce(),
            response.getWidth(),
            response.getFloor(),
            response.getRawGForce()
        ].map({ $0.map({ String($0) }) ?? "-" }).joined(separator: ", ")
    }
    
    func onSerialNumberResponse(_ response: WELDeviceLibrary.SerialNumberResponse) {
        serialNoDetails = response.getSerialNumber() ?? "-"
    }
    
    func onCalibrationDateResponse(_ response: CalibrationDateResponse) {
    }

    func onCheckDateResponse(_ response: CheckDateResponse) {
    }
    
    func onDeviceStatus(_ status: ReadyStatus) {
        isReady = status == .Ready
    }
}
