//
//  ManagerWrapper.swift
//  WELDeviceExample
//

import Foundation
import WELDeviceLibrary

class ManagerWrapper: WELManagerDelegate, ObservableObject {
    
    var manager: WELManager?
    
    #if targetEnvironment(simulator)
    let isSimulated: Bool = true
    #else
    let isSimulated: Bool = false
    #endif
    
    @Published public var message: String = ""
    @Published public var isReady: Bool = false
    @Published public var isScanning: Bool = false
    @Published public var deviceWrappers: [DeviceWrapper] = []
    @Published public var selectedDeviceWrapper: DeviceWrapper?
    
    public init(_ forceMock: Bool = false) {
        let mock = forceMock || isSimulated
        if mock {
            _WELMockHelper.mockBasicSFT()
        }
        
        try? manager = WELManagerBuilder().setDelegate(self).build()
    }
    
    func onManagerState(_ status: WELDeviceLibrary.ReadyStatus) {
        self.isReady = status == .Ready
    }
    
    func onManagerScanningStatus(_ status: WELDeviceLibrary.ScanningStatus) {
        self.isScanning = status == .Scanning
    }
    
    func onSFTDevicesFound(_ device: WELDeviceLibrary.SFTDevice) {
        self.deviceWrappers.append(DeviceWrapper(device))
    }
    
    func onConnectionChanged(_ device: WELDeviceLibrary.WELDevice, _ status: WELDeviceLibrary.ConnectionStatus) {
        if status == .Connected, let sftDevice = deviceWrappers.first(where: { $0.device.identifier == device.identifier }) {
            sftDevice.device.readFirmwareVersion()
        }
    }
    
    func onLog(_ timestamp: Date, _ type: WELDeviceLibrary.LogType, _ message: String) {
        self.message = message
    }
}
